PRO syntool
  ;+
  ; NAME:
  ;       SYNTOOL
  ;
  ; PURPOSE:
  ;       simulates an IRIS observation series
  ;       logics included:
  ;         -reads xml-input files, and copies these files into target-folder if desired
  ;         -reads data input files from simulation; SJI: FUV1, FUV2, NUV1, (NUV2 missing); Spectras: FUV1, FUV2, NUV
  ;         -handles the PZT-logic of the instrument
  ;         -interpolation of input data in all dimensions, except wavelength (i.e. time, x, y)
  ;         -binning of data
  ;         -input data is multiplied by exposure time (assumed input: datacount/s, output: datacount)
  ;         -fits-files are created, data is zeroed outside of CRS-windows
  ;         -FUV, NUV and SJI get each a separate file
  ;         -keywords are included in header
  ;
  ; CALLING SEQUENCE:
  ;       syntool
  ;
  ; INPUT PARAMETER:
  ;       Are so far given within this procedure, see further below
  ;
  ; KEYWORDS:
  ;       none
  ;
  ; OUTPUTS:
  ;       none
  ;
  ; COMMON BLOCKS:
  ;       none
  ;
  ; MODIFICATION HISTORY:
  ;       11-Jun-2012  M. Wiesmann, UIO
  ;       works best with xml-files created by irisplantool_20120928.jar by Ankur Somani (LMSAL)
  ;-



  ; INPUT
  OBSfile = 'OBS-T-00004.xml'   ; The OBS filename
  OBSdir = 'XMLTest/'       ; OBS-file directory
  inputfileFUV = 'Data_cb24bih/iris_synthetic_fuv_####.fits' ; directory and format of input files (as used in fns)
  inputfileNUV = 'Data_cb24bih/iris_synthetic_nuv_cb24bih_###.fits' ; directory and format of input files (as used in fns)
  filemin = 385             ; first file number
  filemax = 410             ; last file number
  dtimeSIM = 10000L         ; time steps between simulation files (in ms)
  SJIframe=2d               ; visible data around SJI (in arcsec), 'Frame'
  maxWINsize=[1400, 1000]   ; max size of plot window [x, y]
  rasterSJI='nu1'           ; ('fu1', 'fu2', 'nu1', 'nu2') choose what SJI data is shown in the overview SJI graph (which will be updated on every step), NUV2 not available yet
  
  
  ;for debugging:
  debug=0         ;set to 1 to control directly, how many pictures will be calculated
  nrpics=3        ;set the number of pictures to be calculated in debug-mode (program stops when this number is reached)
  debugtime=0     ;this time will be added to the real time in the plot output, when in debug-mode (in ms)
  
  ;some keywords
  savefitsfiles=1 ;set to 1 if you want to save the fits files
  saveplot=1      ;set to 1 if you want to save the images, otherwise they will be plotted on screen
  makemovie=1     ;set to 1 if you want to create the movie out of the pictures (saveplot should be 1), qt-tools required (http://omino.com/sw/qt_tools/), MAC only
  playmovie=1     ;set to 1 if you want to play the movie immediately after creating, MAC only
  quicktimeloc='/Applications/QuickTime\ Player.app/Contents/MacOS/QuickTime\'  ;location of your quicktime application, required for playmovie, MAC only
  savelog=1       ;set to 1 if you want to save a logfile
  copyxml=1       ;set to 1 if you want to copy the xml-files to the destination folder
  
  
  ;graph variables
  pcharsize=0.8       ;size of the text
  pcharthick=1        ;thickness of the text
  color_active=[255, 0, 0]      ;color of text, when window is active [r, g, b]
  color_inactive=[5, 180, 150]  ;color of text, when window is inactive [r, g, b]
  threshold_SJI_Overview=0.0002 ;threshold for display used in histo_opt for overview SJI
  threshold_SJI=0.002          ;threshold for display used in histo_opt for SJI(s)
  threshold_Spec=0.02         ;threshold for display used in histo_opt for spectral images
  
  
  ;sizes of CCDs
  PixCCDxfuv1=2072 ;number of pixels on FUV1 CCD in x-direction
  PixCCDyfuv1=1096 ;number of pixels on FUV1 CCD in y-direction
  PixCCDxfuv2=PixCCDxfuv1 ;number of pixels on FUV2 CCD in x-direction
  PixCCDyfuv2=PixCCDyfuv1 ;number of pixels on FUV2 CCD in y-direction
  PixCCDxnuv=PixCCDxfuv1 ;number of pixels on NUV CCD in x-direction
  PixCCDynuv=PixCCDyfuv1 ;number of pixels on NUV CCD in y-direction
  PixCCDxfuvSJI=PixCCDxfuv1/2 ;number of pixels on FUV SJI CCD in x-direction
  PixCCDyfuvSJI=PixCCDyfuv1 ;number of pixels on FUV SJI CCD in y-direction
  PixCCDxnuvSJI=PixCCDxfuv1/2 ;number of pixels on NUV SJI CCD in x-direction
  PixCCDynuvSJI=PixCCDyfuv1 ;number of pixels on NUV SJI CCD in y-direction
  PZTnoChangeValue=9999     ; if all PZT variables in OBS have this value, the position is not changed
  
  
  
  
  ; get current date and time to create a new folder
  if savefitsfiles || saveplot || makemovie || savelog || copyxml then begin
    datenum=bin_date(systime(/utc))
    datestamp=''
    for i=0,5 do begin
      if datenum[i] lt 10 then datestamp=datestamp + '0'
      datestamp = datestamp + strcompress(string(datenum[i]),/remove_all)
      if i lt 5 then datestamp = datestamp + '_'
    endfor
    print, 'saving data in folder: ', datestamp
    spawn, "mkdir "+datestamp
  endif
  
  ;get data from XML-files and save the xml-files to the above created folder
  readtables, OBSfile, OBSdir, OBSList, FRMList, FDBList, CRSList, error, copyxml, datestamp
  
  ; check for errors while reading the XML-files
  if error then begin
    print,'There is something wrong with the given OBS file, exiting program'
    return
  endif
  
  
  
  
  
  
  PZT = [0d, 0d]
  
  for nrOBSrepeats=0,OBSList.Repeat_Obs-1 do begin ; loop over OBS repeats
    print, 'OBS repeat ', nrOBSrepeats+1, ' of ', OBSList.Repeat_Obs
    
    if nrOBSrepeats eq 0 then begin
      ;initialize time and some other values
      time = OBSList.Tref_Obs
      localtime = time
      timeOBSrep = time
      OBScadence = OBSList.Cadence_Obs
      step = 0
      filecur = filemin
    endif else begin
      ;add cadence of OBS to time and check if cadence is not too short
      timeOBSrep = timeOBSrep + OBScadence
      if timeOBSrep gt time then begin
        localtime = localtime + timeOBSrep - time
        time = timeOBSrep
      endif
    endelse
    
    
    for nrFRM=0,OBSList.NumEntries-1 do begin ;loop over whole OBS, different FRMs
    
      if (*(OBSList).Tref_FRM)[nrFRM] gt time then begin
        ; check if time is correct, or has to be adjusted
        localtime = localtime + (*(OBSList).Tref_FRM)[nrFRM] - time
        time = (*(OBSList).Tref_FRM)[nrFRM]
      endif
      
      ; set position of lower left corner of origin, according to PZT
      if (*(OBSList).PZT_A_Abs)[nrFRM] ne PZTnoChangeValue || $
        (*(OBSList).PZT_B_Abs)[nrFRM] ne PZTnoChangeValue || $
        (*(OBSList).PZT_C_Abs)[nrFRM] ne PZTnoChangeValue then begin
        PZT = pzt2xy((*(OBSList).PZT_A_Abs)[nrFRM], (*(OBSList).PZT_B_Abs)[nrFRM], (*(OBSList).PZT_C_Abs)[nrFRM])
        ; PZT = round(PZT*6)
        addstep=0
      endif else begin
        if N_ELEMENTS(PZTStep) ne 0 then PZT = PZT-PZTStep ;reverse last addition of step, because we want to add the current stepsize
        addstep=1
      endelse
      PZTStep = pzt2xy((*(OBSList).PZT_A_Step)[nrFRM], (*(OBSList).PZT_B_Step)[nrFRM], (*(OBSList).PZT_C_Step)[nrFRM])
      ; PZTStep = round(PZTStep*6)
      if addstep then PZT = PZT+PZTStep
      
      ;get current FRM index
      currentFRM=-1
      for i=0,N_ELEMENTS(FRMList)-1 do begin
        if (*(OBSList).FRM_ID)[nrFRM] eq FRMList[i].ID then begin
          currentFRM = i
          break
        endif
      endfor
      if currentFRM eq -1 then begin
        print, 'Could not find corresponding FRM ', (*(OBSList).FRM_ID)[nrFRM]
        return
      endif
      print, 'current FRM index ', currentFRM, '; ID: ', FRMList[currentFRM].ID
      
      
      
      for nrFRMrepeats=0,(*(OBSList).Repeat_FRM)[nrFRM]-1 do begin ; loop over FRM repeats
      
        if nrFRMrepeats eq 0 then begin
          ;get cadence of 1 FRM (cadence given in FRM-file is for all repeats)
          timeFRMrep = time
          FRMcadence = (*(OBSList).Cadence_FRM)[nrFRM]
        endif
        
        
        for nrFDB=0,FRMList[currentFRM].NumEntries-1 do begin ; loop over current FRM, different entries
        
          if (*(FRMList[currentFRM]).Tref)[nrFDB] gt 0 then begin
            ;check time given within FRM-file, and add it to the time
            if nrFDB gt 0 then subtract = (*(FRMList[currentFRM]).Tref)[nrFDB-1] else subtract = 0
            time = time + (*(FRMList[currentFRM]).Tref)[nrFDB] - subtract
            localtime = localtime + (*(FRMList[currentFRM]).Tref)[nrFDB] - subtract
          endif
          
          PZTOFF = pzt2xy((*(FRMList[currentFRM]).PZT_A_Rel)[nrFDB], (*(FRMList[currentFRM]).PZT_B_Rel)[nrFDB], (*(FRMList[currentFRM]).PZT_C_Rel)[nrFDB])
          ; PZTOFF = round(PZTOFF*6)
          PZTcurrent = PZT + PZTOFF
          
          ;get current FDB indices
          currentFDB=[-1, -1, -1, -1] ;[sji, nuv, fuv1, fuv2]
          noFDB=0
          if (*(FRMList[currentFRM]).SJI_FDB_ID)[nrFDB] eq '0' then noFDB = noFDB + 1
          if (*(FRMList[currentFRM]).NUV_SG_FDB_ID)[nrFDB] eq '0' then noFDB = noFDB + 1
          if (*(FRMList[currentFRM]).FUV1_SG_FDB_ID)[nrFDB] eq '0' then noFDB = noFDB + 1
          if (*(FRMList[currentFRM]).FUV2_SG_FDB_ID)[nrFDB] eq '0' then noFDB = noFDB + 1
          for i=0,N_ELEMENTS(FDBList)-1 do begin
            if (*(FRMList[currentFRM]).SJI_FDB_ID)[nrFDB] ne '0' then $
              if (*(FRMList[currentFRM]).SJI_FDB_ID)[nrFDB] eq FDBList[i].ID then currentFDB[0] = i
            if (*(FRMList[currentFRM]).NUV_SG_FDB_ID)[nrFDB] ne '0' then $
              if (*(FRMList[currentFRM]).NUV_SG_FDB_ID)[nrFDB] eq FDBList[i].ID then currentFDB[1] = i
            if (*(FRMList[currentFRM]).FUV1_SG_FDB_ID)[nrFDB] ne '0' then $
              if (*(FRMList[currentFRM]).FUV1_SG_FDB_ID)[nrFDB] eq FDBList[i].ID then currentFDB[2] = i
            if (*(FRMList[currentFRM]).FUV2_SG_FDB_ID)[nrFDB] ne '0' then $
              if (*(FRMList[currentFRM]).FUV2_SG_FDB_ID)[nrFDB] eq FDBList[i].ID then currentFDB[3] = i
          endfor
          index = WHERE(currentFDB eq -1, count)
          if (count eq 4) || (count ne noFDB) then begin
            print, 'Could not find one or more corresponding FDB in this FRM ', FRMList[currentFRM].ID
            return
          endif
          ;print, 'current FDB indices: ', currentFDB
          
          
          ;get current CRS indices
          currentCRS=[-1, -1, -1, -1] ;[sji, nuv, fuv1, fuv2]
          for i=0,N_ELEMENTS(CRSList)-1 do begin
            for j=0,3 do begin
              if currentFDB[j] ge 0 then $
                if FDBList[currentFDB[j]].CRSID eq CRSList[i].ID then currentCRS[j] = i
            endfor
          endfor
          index = WHERE(currentCRS eq -1, count)
          if (count eq 4) || (count ne noFDB) then begin
            print, 'Could not find one or more corresponding CRS'
            return
          endif
          ;print, 'current CRS indices: ', currentCRS
          
          
          
          if (step eq 0) || (localtime ge 2*dtimeSIM) then begin
            ;initial values, files
            ;FUV
            a = long(localtime/dtimeSIM)
            localtime = localtime MOD dtimeSIM
            filecur = filecur + a
            if filecur gt filemax then filecur = filemin
            file=fns(inputfileFUV,filecur)
            print, 'loading new data file: ', file
            sim1=file
            fxread, file, fuvSJI1, header
            fxbopen,unit,file,1,header
            fxbread,unit,fuv11,'FUV 1'
            fxbread,unit,fuv12,'FUV 2'
            fxbclose,unit
            
            filecur = filecur + 1
            if filecur gt filemax then filecur = filemin
            file=fns(inputfileFUV,filecur)
            print, 'loading new data file: ', file
            sim2=file
            fxread, file, fuvSJI2, header
            fxbopen,unit,file,1,header
            fxbread,unit,fuv21,'FUV 1'
            fxbread,unit,fuv22,'FUV 2'
            fxbclose,unit
            
            ;NUV
            filecur = filecur - 1
            file=fns(inputfileNUV,filecur)
            print, 'loading new data file: ', file
            sim3=file
            nuvSJI1 = readfits(file, header1, /silent)
            nuv1lambda = readfits(file, header11, exten_no=1, /silent)
            nuv1 = readfits(file, header12, exten_no=2, /silent)
            
            filecur = filecur + 1
            file=fns(inputfileNUV,filecur)
            print, 'loading new data file: ', file
            sim4=file
            nuvSJI2 = readfits(file, header2, /silent)
            nuv2lambda = readfits(file, header21, exten_no=1, /silent)
            nuv2 = readfits(file, header22, exten_no=2, /silent)
          endif else begin
            if localtime ge dtimeSIM then begin
              ;load a new simulation file every 10 seconds
              ;FUV
              localtime = localtime-dtimeSIM
              fuvSJI1=fuvSJI2
              fuv11=fuv21
              fuv12=fuv22
              filecur=filecur+1
              if filecur gt filemax then filecur = filemin
              file=fns(inputfileFUV,filecur)
              print, 'loading new data file: ', file
              sim1=sim2
              sim2=file
              fxread, file, fuvSJI2, header
              fxbopen,unit,file,1,header
              fxbread,unit,fuv21,'FUV 1'
              fxbread,unit,fuv22,'FUV 2'
              fxbclose,unit
              
              ;NUV
              nuvSJI1=nuvSJI2
              nuv1lambda=nuv2lambda
              nuv1=nuv2
              file=fns(inputfileNUV,filecur)
              print, 'loading new data file: ', file
              sim3=sim4
              sim4=file
              nuvSJI2 = readfits(file, header2, /silent)
              nuv2lambda = readfits(file, header21, exten_no=1, /silent)
              nuv2 = readfits(file, header22, exten_no=2, /silent)
            endif
          endelse
          
          
          if step eq 0 then begin
            ; initialize window parameters
            !p.charsize=pcharsize
            !p.charthick=pcharthick
            
            initGraph, OBSList, FRMList, FDBList, CRSList, round(SJIframe*6), PZTnoChangeValue, PixCCDxfuv1, PixCCDxfuvSJI, $
              SpecData, SJIData, WINsize, PosClock, xmax=maxWINsize[0], ymax=maxWINsize[1]
            
            frozenSlit=intarr(N_ELEMENTS(SJIData))
            
            ;create logfile
            if savelog then begin
              logFilename=datestamp+'/AAlog.txt'
              OpenW, logFileUnit, logFilename, /get_lun
              log='Log of syntool.pro, OBS ID: '+OBSList.ID
              printf,logFileUnit, log
              log='Time [ms];PZTx;PZTy;OBSrep;OBSentry;FRMrep;FRMentry;FW;FRM ID;FDB ID SJI;FDB ID FUV; FDB ID NUV;CRS ID SJI;CRS ID FUV;CRS ID NUV;'+ $
              'Sim File 1 FUV;Sim File 2 FUV;Sim File 1 NUV; Sim File 2 NUV'
              printf,logFileUnit, log
              Close, logFileUnit
            endif
            
          endif
          
          ;write to the log file
          if savelog then begin
            OpenU, logFileUnit, logFilename, /append
            if currentFDB[0] ge 0 then fdbsji=FDBList[currentFDB[0]].ID else fdbsji='-'
            if currentFDB[2] ge 0 then fdbfuv=FDBList[currentFDB[2]].ID else fdbfuv='-'
            if currentFDB[1] ge 0 then fdbnuv=FDBList[currentFDB[1]].ID else fdbnuv='-'
            if currentCRS[0] ge 0 then crssji=CRSList[currentCRS[0]].ID else crssji='-'
            if currentCRS[2] ge 0 then crsfuv=CRSList[currentCRS[2]].ID else crsfuv='-'
            if currentCRS[1] ge 0 then crsnuv=CRSList[currentCRS[1]].ID else crsnuv='-'
            log=string(time)+';'+$
            string(PZTcurrent[0])+';'+$
            string(PZTcurrent[1])+';'+$
            string(nrOBSrepeats+1)+';'+$
            string(nrFRM+1)+';'+$
            string(nrFRMrepeats+1)+';'+$
            string(nrFDB+1)+';'+$
            string((*(FRMList[currentFRM]).FW)[nrFDB])+';'+$
            FRMList[currentFRM].ID+';'+$
            fdbsji+';'+$
            fdbfuv+';'+$
            fdbnuv+';'+$
            crssji+';'+$
            crsfuv+';'+$
            crsnuv+';'+$
            sim1+';'+$
            sim2+';'+$
            sim3+';'+$
            sim4
            log=strcompress(log,/remove_all)
            printf, logFileUnit, log
            Close, logFileUnit
          endif
          
          
          
          
          
          ;timefactor for interpolation between time steps
          timefac = double(localtime)/dtimeSIM
          
          SpecwinUpdate=intarr(N_ELEMENTS(SpecData))
          
          ;calculate FUV if necessary
          if currentCRS[2] ge 0 then begin
            ;interpolation in time, x- and y direction
            fuv1 = DataPrepareSpec(fuv11, fuv21, PixCCDxfuv1, PixCCDyfuv1, PZTcurrent, timefac, /rightpad)
            fuv2 = DataPrepareSpec(fuv12, fuv22, PixCCDxfuv2, PixCCDyfuv2, PZTcurrent, timefac)
            ;put fuv1 and fuv2 into 1 matrix
            fuv = fltarr((size(fuv1))[1]+(size(fuv2))[1], max([(size(fuv1))[2], (size(fuv2))[2]]))
            fuv[0:(size(fuv1))[1]-1, 0: (size(fuv1))[2]-1] = fuv1
            fuv[(size(fuv1))[1]:(size(fuv1))[1]+(size(fuv2))[1]-1, 0:(size(fuv2))[2]-1] = fuv2
            
            ;check if we need to rebin the data
            sumspec=CRSList[currentCRS[2]].Spectral
            sumspat=CRSList[currentCRS[2]].Spatial
            if (sumspec gt 1) || (sumspat gt 1) then $
              fuv = rebin(fuv, (PixCCDxfuv1+PixCCDxfuv2)/sumspec, max([PixCCDyfuv1, PixCCDxfuv2])/sumspat) * sumspec * sumspat
              
            ;multiply with defined exposure duratioin
            fuv = fuv * FDBList[currentFDB[2]].Exp_Duration/1000d
            
            ; set all data to zero, except for CRS windows
            fuvtemp = fltarr((size(fuv))[1],(size(fuv))[2])
            for i=0,CRSList[currentCRS[2]].SubRegions-1 do begin
              startR = (*(CRSList[currentCRS[2]]).StartRow)[i]
              endR = startR - 1 + (*(CRSList[currentCRS[2]]).NumRows)[i]
              startC = (*(CRSList[currentCRS[2]]).StartCol)[i]
              endC = startC - 1 + (*(CRSList[currentCRS[2]]).NumCols)[i]
              ;adjust for rebinning
              if sumspec gt 1 then begin
                startR = startR / sumspec
                endR = endR / sumspec
              endif
              if sumspat gt 1 then begin
                startC = startC / sumspat
                endC = endC / sumspat
              endif
              fuvtemp[startR:endR,startC:endC] = fuv[startR:endR,startC:endC]
              
              ;set data of region to structure PlotData for later plotting
              for pda=0,N_ELEMENTS(SpecData)-1 do begin
                if (SpecData[pda].CRSindex eq currentCRS[2]) && (SpecData[pda].SubRegionID eq (*(CRSList[currentCRS[2]]).SubRegionID)[i]) then begin
                  *(SpecData[pda]).data = fuv[startR:endR,startC:endC]
                  SpecwinUpdate[pda]=1
                  
                  ;the first time this spectral window is populated, we need to set the minium and maximum value for scaling
                  if (SpecData[pda].min_show eq 0) && (SpecData[pda].max_show eq 0) then begin
                    temp = Histo_Opt(*(SpecData[pda]).data, threshold_Spec)
                    SpecData[pda].max_show = max(temp, min=m)
                    SpecData[pda].min_show = m
                  endif
                endif
              endfor
            endfor
            fuv = fuvtemp
            
          endif ;FUV calculation
          
          
          
          ;calculate NUV if necessary
          if currentCRS[1] ge 0 then begin
            ;interpolation in time, x- and y direction
            nuv = DataPrepareSpec(nuv1, nuv2, PixCCDxnuv, PixCCDynuv, PZTcurrent, timefac, /rightpad, /nuv)
            
            ;check if we need to rebin the data
            sumspec=CRSList[currentCRS[1]].Spectral
            sumspat=CRSList[currentCRS[1]].Spatial
            if (sumspec gt 1) || (sumspat gt 1) then $
              nuv = rebin(nuv, PixCCDxnuv/sumspec, PixCCDynuv/sumspat) * sumspec * sumspat
              
            ;multiply with defined exposure duration
            nuv = nuv * FDBList[currentFDB[1]].Exp_Duration/1000d
            
            ; set all data to zero, except for CRS windows
            nuvtemp = fltarr((size(nuv))[1],(size(nuv))[2])
            for i=0,CRSList[currentCRS[1]].SubRegions-1 do begin
              startR = (*(CRSList[currentCRS[1]]).StartRow)[i]
              if startR gt PixCCDxfuv1 then startR = startR - PixCCDxfuv1
              endR = startR - 1 + (*(CRSList[currentCRS[1]]).NumRows)[i]
              startC = (*(CRSList[currentCRS[1]]).StartCol)[i]
              endC = startC - 1 + (*(CRSList[currentCRS[1]]).NumCols)[i]
              ;adjust for rebinning
              if sumspec gt 1 then begin
                startR = startR / sumspec
                endR = endR / sumspec
              endif
              if sumspat gt 1 then begin
                startC = startC / sumspat
                endC = endC / sumspat
              endif
              nuvtemp[startR:endR,startC:endC] = nuv[startR:endR,startC:endC]
              
              ;set data of region to structure PlotData for later plotting
              for pda=0,N_ELEMENTS(SpecData)-1 do begin
                if (SpecData[pda].CRSindex eq currentCRS[1]) && (SpecData[pda].SubRegionID eq (*(CRSList[currentCRS[1]]).SubRegionID)[i]) then begin
                  *(SpecData[pda]).data = nuv[startR:endR,startC:endC]
                  SpecwinUpdate[pda]=1
                  
                  ;the first time this spectral window is populated, we need to set the minium and maximum value for scaling
                  if (SpecData[pda].min_show eq 0) && (SpecData[pda].max_show eq 0) then begin
                    temp = Histo_Opt(*(SpecData[pda]).data, threshold_Spec)
                    SpecData[pda].max_show = max(temp, min=m)
                    SpecData[pda].min_show = m
                  endif
                endif
              endfor
            endfor
            nuv = nuvtemp
            
          endif ;NUV calculation
          
          
          
          ;SJI calculation
          SJIwinUpdate=intarr(N_ELEMENTS(SJIData)) ;indicates whether a specific SJI window is updated
          SJIwinUpdate[0]=1 ;Overview SJI window is always updated
          if currentCRS[0] ge 0 then begin
            for i=1,N_ELEMENTS(SJIData)-1 do begin
              if SJIData[i].CRSindex eq currentCRS[0] then begin
                SJIind = i
                SJIwinUpdate[i]=1
                frozenSlit[i]=PixCCDxfuvSJI/2
                break
              endif
            endfor
          endif else SJIind=-1
          DataPrepareSJI, fuvSJI1, fuvSJI2, nuvSJI1, nuvSJI2, rasterSJI, PixCCDxfuvSJI, PixCCDyfuvSJI, timefac, PZTcurrent, SJIData, SJIind, SJI, $
            threshold_SJI_Overview, threshold_SJI
          currentBox=SJIData[0].Showcoords
          currentBox[0]=currentBox[0]+PZTcurrent[0]*6
          currentBox[4]=currentBox[4]+PZTcurrent[0]*6
          currentBox[2]=currentBox[2]+PZTcurrent[1]*6
          currentBox[5]=currentBox[5]+PZTcurrent[1]*6
          currentSlit=PixCCDxfuvSJI/2+PZTcurrent[0]*6
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          if savefitsfiles then begin
          
            if currentCRS[0] ge 0 then begin
              file=datestamp + fns('/AIA_####',step) + '_SJI.fits'
              fxhmake, header, SJI, /initialize, /extend, /date
              SyntoolAddKeywords, header, OBSList, FRMList, FDBList, CRSList, $
                0, currentFRM, currentFDB[0], currentCRS[0], $
                nrOBSrepeats, nrFRM, nrFRMrepeats, nrFDB, $
                time, PZT, $
                sim1, sim2, sim3, sim4
              fxwrite, file, header, SJI
            endif
            
            if currentCRS[1] ge 0 then begin
              file=datestamp + fns('/AIA_####',step) + '_NUV.fits'
              fxhmake, header, nuv, /initialize, /extend, /date
              SyntoolAddKeywords, header, OBSList, FRMList, FDBList, CRSList, $
                1, currentFRM, currentFDB[1], currentCRS[1], $
                nrOBSrepeats, nrFRM, nrFRMrepeats, nrFDB, $
                time, PZT, $
                sim1, sim2, sim3, sim4
              fxwrite, file, header, nuv
            endif
            
            if currentCRS[2] ge 0 then begin
              file=datestamp + fns('/AIA_####',step) + '_FUV.fits'
              ; Create a primary header and write it out
              fxhmake, header, fuv, /initialize, /extend, /date
              SyntoolAddKeywords, header, OBSList, FRMList, FDBList, CRSList, $
                2, currentFRM, currentFDB[2], currentCRS[2], $
                nrOBSrepeats, nrFRM, nrFRMrepeats, nrFDB, $
                time, PZT, $
                sim1, sim2, sim3, sim4
              fxwrite, file, header, fuv
            endif
            
          endif ;save fits files
          
          
          
          
          
          ;;; graphical output
          
          if step eq 0 then begin
            if saveplot then set_plot,'z' else set_plot,'x'
            !p.background=253
            !p.color=0
            !p.ticklen=-.02
            ;!p.charsize=1  ;those 2 are defined above
            ;!p.charthick=1
            if saveplot then device,set_res=WINsize $
            else window, 16, xs=WINsize[0], ys=WINsize[1]
            ;load color table, works only with set_plot,'x', or see 3 lines below
            loadct,0
            ;reverse black and white, works like this only when plotting directly on screen, not in file
            ;but like this it works: tvrd(),r,g,b
            tvlct,r,g,b,/get
            ;r=reverse(r)
            ;g=reverse(g)
            ;b=reverse(b)
            ;active color
            r[255]=color_active[0]
            g[255]=color_active[1]
            b[255]=color_active[2]
            ;non-active color
            r[254]=color_inactive[0]
            g[254]=color_inactive[1]
            b[254]=color_inactive[2]
            tvlct,r,g,b
          endif
          
          
          
          
          ;plot SJI
          for i=0,N_ELEMENTS(SJIData)-1 do begin
            if SJIwinUpdate[i] then colortext=255 else colortext=254
            bold=1
            titlesize=1.0*!p.charsize
            titleposy=titlesize*!d.y_ch_size/WINsize[1]
            datasize=0.9d*!p.CHARSIZE
            dataposy=datasize*!d.y_ch_size/WINsize[1]
            leftpos=0.1*!d.x_ch_size/WINsize[0]
            
            if SJIData[i].min_show eq SJIData[i].max_show then maxshow=SJIData[i].max_show+1e-10 else maxshow=SJIData[i].max_show
            
            if i eq 0 then begin
              pih, *(SJIData[i]).data, threshold_SJI_Overview, position=SJIData[i].WINcoords, $
                origin=[SJIData[i].Boxcoords[0]+1, SJIData[i].Boxcoords[2]+1], $
                min=SJIData[i].min_show, max=maxshow, $
                ytit='Solar Y [Px]', xtit='Solar X [Px]', $
                xstyle=9, ystyle=9, top=253
              axis,xaxis=1,xrange=[(SJIData[i].Boxcoords[0])/6.0, (SJIData[i].Boxcoords[4])/6.0], xtit='Solar X [arcsec]', xstyle=1
              axis,yaxis=1,yrange=[SJIData[i].Boxcoords[2]/6.0, SJIData[i].Boxcoords[5]/6.0], ytit='Solar Y [arcsec]', ystyle=1
              oplot, [currentBox[0], currentBox[0]], [currentBox[2], currentBox[5]], thick=2, col=128
              oplot, [currentBox[4], currentBox[4]], [currentBox[2], currentBox[5]], thick=2, col=128
              oplot, [currentBox[0], currentBox[4]], [currentBox[2], currentBox[2]], thick=2, col=128
              oplot, [currentBox[0], currentBox[4]], [currentBox[5], currentBox[5]], thick=2, col=128
              oplot, [currentSlit, currentSlit], [SJIData[i].Showcoords[2]+PZTcurrent[1]*6, SJIData[i].Showcoords[5]+PZTcurrent[1]*6], thick=2, col=128
              
            endif else if i eq 1 then begin
              pih,*(SJIData[i]).data, threshold_SJI, position=SJIData[i].WINcoords, $
                ytit='Solar Y [Px]', xtit='Solar X [Px]', $
                min=SJIData[i].min_show, max=maxshow, $
                origin=[SJIData[i].Boxcoords[0]+1, SJIData[i].Boxcoords[2]+1], scale=[SJIData[i].sum_spatX, SJIData[i].sum_spatY], $
                xstyle=9, ystyle=9, top=253, /noerase
              axis,xaxis=1,xrange=[(PixCCDxfuvSJI/2-(SJIData[i].Boxcoords[0]))/6.0-SJIData[i].PZTx, (PixCCDxfuvSJI/2-(SJIData[i].Boxcoords[4]))/6.0-SJIData[i].PZTx], xtit='Solar X [arcsec]', xstyle=1
            endif else begin
              pih,*(SJIData[i]).data, threshold_SJI, position=SJIData[i].WINcoords, $
                origin=[SJIData[i].Boxcoords[0]+1, SJIData[i].Boxcoords[2]+1], scale=[SJIData[i].sum_spatX, SJIData[i].sum_spatY], $
                min=SJIData[i].min_show, max=maxshow, $
                ;xtit='Solar X [Px]', $
                xstyle=9, ystyle=5, /noerase, top=253
              axis,xaxis=1,xrange=[(PixCCDxfuvSJI/2-(SJIData[i].Boxcoords[0]))/6.0-SJIData[i].PZTx, (PixCCDxfuvSJI/2-(SJIData[i].Boxcoords[4]))/6.0-SJIData[i].PZTx], xtit='Solar X [arcsec]', xstyle=1
            endelse
            
            if (i eq N_ELEMENTS(SJIData)-1) && (i gt 0) then $
              axis,yaxis=1,yrange=[(SJIData[i].Boxcoords[2]-PixCCDynuvSJI/2)/6.0-SJIData[i].PZTy, (SJIData[i].Boxcoords[5]-PixCCDynuvSJI/2)/6.0-SJIData[i].PZTy], ytit='Solar Y [arcsec]', ystyle=1
              
            ;draw slit if necessary
            if frozenSlit[i] gt 0 then begin
              oplot, [frozenSlit[i], frozenSlit[i]], [SJIData[i].BOXcoords[2], SJIData[i].BOXcoords[5]], thick=2, col=128
            endif
            
            ;write title and some values
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[3]-1.2*titleposy, SJIData[i].name, color=colortext, charsize=titlesize, charthick=bold, /normal
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[3]-2.4*titleposy, SJIData[i].name2, color=colortext, charsize=titlesize, charthick=bold, /normal
            if i eq 0 then begin
              text1='X '+string(PZT[0],format='(f5.2)')
              text2='Y '+string(PZT[1],format='(f5.2)')
            endif else begin
              maxval=string(max(*(SJIData[i]).data),format='(e7.1)')
              meanval=string(mean(*(SJIData[i]).data),format='(e7.1)')
              text1='A '+meanval
              text2='M '+maxval
            endelse
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[1]+1.5*dataposy, text1, color=colortext, charsize=datasize, charthick=bold, /normal
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[1]+0.3*dataposy, text2, color=colortext, charsize=datasize, charthick=bold, /normal
          endfor
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          ;plot Spectra
          for i=0,N_ELEMENTS(SpecData)-1 do begin
            if SpecwinUpdate[i] then colortext=255 else colortext=254
            maxval=string(max(*(SpecData[i]).data),format='(e7.1)')
            meanval=string(mean(*(SpecData[i]).data),format='(e7.1)')
            if SpecData[i].min_show eq SpecData[i].max_show then maxshow=SpecData[i].max_show+1e-10 else maxshow=SpecData[i].max_show
            case SpecData[i].type of
              'fu1': begin
                displambda=0.01298
                startlambda=1331.6
              end
              'fu2': begin
                displambda=0.01272
                startlambda=1380.6 - displambda * PixCCDxfuv1
              end
              'nuv': begin
                displambda=0.02546
                startlambda=2782.6 - displambda * PixCCDxfuv1
              end
            endcase
            if i eq 0 then begin
              pih,*(SpecData[i]).data*(-1d), threshold_Spec, position=SpecData[i].WINcoords, $
                origin=[SpecData[i].SIMcoords[0]+1, SpecData[i].SIMcoords[2]+1], scale=[SpecData[i].sum_spec, SpecData[i].sum_spat], $
                max=SpecData[i].min_show*(-1d), min=maxshow*(-1d), $
                xtit=Greek('lambda')+' [Px]', ytit='Solar Y [Px]', xstyle=9, ystyle=9, xticks=SpecData[i].nrticks, $
                /noerase, top=253;, noscale=1
              axis,xaxis=1,xrange=[startlambda+displambda*SpecData[i].SIMcoords[0], startlambda+displambda*SpecData[i].SIMcoords[4]],xtit=Greek('lambda')+' [!3' + String("305B) + '!X]', xstyle=1, xticks=SpecData[i].nrticks, xtickformat='(i4)'
            endif else if i eq N_ELEMENTS(SpecData)-1 then begin
              pih,*(SpecData[i]).data*(-1d), threshold_Spec,position=SpecData[i].WINcoords, $
                origin=[SpecData[i].SIMcoords[0]+1, SpecData[i].SIMcoords[2]+1], scale=[SpecData[i].sum_spec, SpecData[i].sum_spat], $
                max=SpecData[i].min_show*(-1d), min=maxshow*(-1d), $
                xtit=Greek('lambda')+' [Px]', xstyle=9, ystyle=5, xticks=SpecData[i].nrticks, $
                /noerase, top=253
              axis,xaxis=1,xrange=[startlambda+displambda*SpecData[i].SIMcoords[0], startlambda+displambda*SpecData[i].SIMcoords[4]],xtit=Greek('lambda')+' [!3' + String("305B) + '!X]', xstyle=1, xticks=SpecData[i].nrticks, xtickformat='(i4)'
              axis,yaxis=1,yrange=[(SpecData[i].SIMcoords[2]-PixCCDynuv/2)/6.0, (SpecData[i].SIMcoords[5]-PixCCDynuv/2)/6.0],ytit='Solar Y [arcsec]', ystyle=1
            endif else begin
              pih,*(SpecData[i]).data*(-1d), threshold_Spec, position=SpecData[i].WINcoords, $
                origin=[SpecData[i].SIMcoords[0]+1, SpecData[i].SIMcoords[2]+1], scale=[SpecData[i].sum_spec, SpecData[i].sum_spat], $
                max=SpecData[i].min_show*(-1d), min=maxshow*(-1d), $
                xtit=Greek('lambda')+' [Px]', xstyle=9, ystyle=5, xticks=SpecData[i].nrticks, $
                /noerase, top=253
              axis,xaxis=1,xrange=[startlambda+displambda*SpecData[i].SIMcoords[0], startlambda+displambda*SpecData[i].SIMcoords[4]],xtit=Greek('lambda')+' [!3' + String("305B) + '!X]', xstyle=1, xticks=SpecData[i].nrticks, xtickformat='(i4)'
            endelse
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[3]-1.2*titleposy, SpecData[i].name, color=colortext, charsize=titlesize, charthick=bold, /normal
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[3]-2.4*titleposy, SpecData[i].name2, color=colortext, charsize=titlesize, charthick=bold, /normal
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[1]+1.5*dataposy, 'A '+meanval, color=colortext, charsize=datasize, charthick=bold, /normal
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[1]+0.3*dataposy, 'M '+maxval, color=colortext, charsize=datasize, charthick=bold, /normal
          endfor
          
          
          ;display time, digitally and analogously, digitally, now in title of SJI
          if debug then stringtime = strtime(time+debugtime) else stringtime = strtime(time)
          if debug then stringtimesec = strtimesec(time+debugtime) else stringtimesec = strtimesec(time)
          xyouts, 0.99, 1-1.7*titleposy, 'Time: '+stringtime, color=0, /normal, charsize=titlesize, charthick=bold, alignment=1, width=w1
          xyouts, 0.98-w1, 1-1.7*titleposy, 'Time [s]: '+stringtimesec, color=0, /normal, charsize=titlesize, charthick=bold, alignment=1, width=w2
          xyouts, 0.97-w1-w2, 1-1.7*titleposy, 'Index: '+string(step,format='(I6.4)'), color=0, /normal, charsize=titlesize, charthick=bold, alignment=1
          clock, stringtime, pos=PosClock[0:1], size=PosClock[2:3], col=255
          
          
          
          
          ; save image
          if saveplot then begin
            file=datestamp + '/' + fns('AApic_####',step) + '.png'
            write_png,file,tvrd(),r,g,b
          endif
          
          
          
          
          
          print, 'step: ', strcompress(string(step), /remove_all), $
            '  time: ', strcompress(string(time), /remove_all), $
            ;'  localtime: ', strcompress(string(localtime), /remove_all), $
            '  PZT: ', strcompress(string(PZT[0]), /remove_all), $
            ' / ', strcompress(string(PZT[1]), /remove_all)
            
          step = step + 1
          if debug && (step eq nrpics) then break
        endfor ;nrFDB=0,FRMList[useFRM].NumEntries-1
        
        ;change the position of the origin
        PZT = PZT + PZTStep
        
        ;add a FRM cadence to time
        timeFRMrep = timeFRMrep + FRMcadence
        if timeFRMrep gt time then begin
          localtime = localtime + timeFRMrep - time
          time = timeFRMrep
        endif
        
        if debug && (step eq nrpics) then break
      endfor ;nrFRMrepeats=0,(*(OBSList).Repeat_FRM)[nrFRM]-1
      
      if debug && (step eq nrpics) then break
    endfor ;nrFRM=0,OBSList.NumEntries-1
    
    if debug && (step eq nrpics) then break
  endfor ;nrOBSrepeats=0,OBSList.Repeat_Obs-1
  
  
  if makemovie then begin
    cd,datestamp
    framerate='--sequencerate=30'
    print,'/usr/local/bin/qt_export --replacefile --video=png,30,100 '+framerate+' '+fns('AApic_####.png',0)+' AAmovie.mov'
    spawn,'/usr/local/bin/qt_export --replacefile --video=png,30,100 '+framerate+' '+fns('AApic_####.png',0)+' AAmovie.mov'
    if playmovie then begin
      print,'opening movie...'
      ;spawn,'/Applications/Utilities/Quicktime\ Player\ 7.app/Contents/MacOS/Quicktime\ Player\ 7 *.mov &'
      spawn, quicktimeloc + ' Player *.mov &'
    endif
    cd,'..'
  endif
  
  print, 'End Time: ', time
  
  if savelog then begin
    OpenU, logFileUnit, logFilename, /append
    log=string(time)
    log=strcompress(log,/remove_all)
    printf, logFileUnit, log
    Close, logFileUnit
    free_lun, logFileUnit
  endif
  
  set_plot,'x'
  !P.Multi=0
end
